@@ -26,7 +26,7 @@ This module contains functions associated to raster images.
 import copy
 import logging
 import os
-from typing import List, Tuple, Union
+from typing import Dict, List, Tuple, Union
 
 # Third party imports
 import numpy as np
@@ -49,8 +49,11 @@ def read_image(path: str, band: int = 1) -> np.ndarray:
     Read image as array
 
     :param path: path
+    :type path: str
     :param band: numero of band to extract
+    :type band: int
     :return: band array
+    :rtype: np.ndarray
     """
     img_ds = rasterio.open(path)
     data = img_ds.read(band)
@@ -61,14 +64,18 @@ def pix_to_coord(
     transform_array: Union[List, np.ndarray],
     row: Union[int, np.ndarray],
     col: Union[List, np.ndarray],
-) -> Union[Tuple[Tuple[np.ndarray, np.ndarray], float, float]]:
+) -> Tuple[np.ndarray, np.ndarray]:
     """
     Transform pixels to coordinates
 
     :param transform_array: Transform
+    :type transform_array: List or np.ndarray
     :param row: row
+    :type row: int or np.ndarray
     :param col: column
+    :type col: List or np.ndarray
     :return: x,y
+    :rtype: np.ndarray, np.ndarray
     """
     transform = Affine.from_gdal(
         transform_array[0],
@@ -78,8 +85,8 @@ def pix_to_coord(
         transform_array[4],
         transform_array[5],
     )
-
-    x, y = rasterio.transform.xy(transform, row, col, offset="center")
+    # Set the offset to ul (upper left)
+    x, y = rasterio.transform.xy(transform, row, col, offset="ul")
 
     if not isinstance(x, int):
         x = np.array(x)
@@ -95,9 +102,13 @@ def reproject_dataset(
     Reproject dataset, and return the corresponding xarray.DataSet
 
     :param dataset: Dataset to reproject
+    :type dataset: xr.Dataset
     :param from_dataset: Dataset to get projection from
+    :type from_dataset: xr.Dataset
     :param interp: interpolation method
+    :type interp: str
     :return: reprojected dataset
+    :rtype: xr.Dataset
     """
 
     # Copy dataset
@@ -161,7 +172,7 @@ def reproject_dataset(
 def read_img(
     img: str,
     no_data: float = None,
-    ref: str = "WGS84",
+    georef_grid: str = "WGS84",
     geoid_path: Union[str, None] = None,
     zunit: str = "m",
     load_data: bool = False,
@@ -170,14 +181,21 @@ def read_img(
     Read image and transform and return the corresponding xarray.DataSet
 
     :param img: Path to the image
+    :type img: str
     :param no_data: no_data value in the image
-    :param ref: WGS84 or geoid
+    :type no_data: float
+    :param ref_georef_grid: WGS84 or geoid
+    :type ref_georef_grid: str
     :param geoid_path: optional, path to local geoid
+    :type ref_georef_grid: str or None
     :param zunit: unit
+    :type zunit: str
     :param load_data: load as dem
+    :type load_data: bool
     :return: dataset containing the variables :
             - im : 2D (row, col) xarray.DataArray float32
             - trans 1D xarray.DataArray float32
+    :rtype: xr.Dataset
     """
     img_ds = rasterio.open(img)
     data = img_ds.read(1)
@@ -188,7 +206,7 @@ def read_img(
         transform,
         img,
         no_data=no_data,
-        ref=ref,
+        georef_grid=georef_grid,
         geoid_path=geoid_path,
         zunit=zunit,
         load_data=load_data,
@@ -202,7 +220,7 @@ def create_dataset(
     transform: np.ndarray,
     img: str,
     no_data: float = None,
-    ref: str = "WGS84",
+    georef_grid: str = "WGS84",
     geoid_path: Union[str, None] = None,
     zunit: str = "m",
     load_data: bool = False,
@@ -212,19 +230,27 @@ def create_dataset(
     and return the corresponding xarray.DataSet
 
     :param data: image data
+    :type data: np.ndarray
     :param transform: image data
+    :type transform: np.ndarray
     :param img: image path
+    :type img: str
     :param no_data: no_data value in the image
-    :param ref: WGS84 or geoid
+    :type no_data: float or None
+    :param georef_grid: WGS84 or geoid
+    :type georef_grid: str
     :param geoid_path: optional path to local geoid, default is EGM96
+    :type geoid_path: str or None
     :param zunit: unit
+    :type zunit: str
     :param load_data: load as dem
+    :type load_data: bool
     :return: xarray.DataSet containing the variables :
             - im : 2D (row, col) xarray.DataArray float32
+    :type data: xr.Dataset
     """
 
     img_ds = rasterio.open(img)
-    georef = img_ds.crs
 
     # Manage nodata
     if no_data is None:
@@ -262,8 +288,11 @@ def create_dataset(
     dataset.coords["trans_len"] = trans_len
     dataset["trans"] = xr.DataArray(data=transform, dims=["trans_len"])
 
+    # The image's crs is used as georef. The georef_grid
+    # argument is used to add an offset if necessary
+    img_crs = img_ds.crs
     # get plani unit
-    if georef.is_geographic:
+    if img_crs.is_geographic:
         plani_unit = u.deg
     else:
         plani_unit = u.m
@@ -273,7 +302,7 @@ def create_dataset(
     dataset.attrs = {
         "no_data": no_data,
         "input_img": img,
-        "georef": georef,
+        "georef": img_crs,
         "xres": transform[1],
         "yres": transform[5],
         "plani_unit": plani_unit,
@@ -281,7 +310,8 @@ def create_dataset(
     }
 
     if load_data is not False:
-        if ref == "geoid":
+        # If the georef is geoid, add geoid offset
+        if georef_grid == "geoid":
             # transform to ellipsoid
             geoid_offset = get_geoid_offset(dataset, geoid_path)
             dataset["im"].data += geoid_offset
@@ -299,10 +329,14 @@ def read_img_from_array(
     If from_dataset is None defaults attributes are set.
 
     :param img_array: array
-    :param no_data: no_data value in the image
+    :type img_array: np.ndarray
     :param from_dataset: dataset to copy
+    :type from_dataset: xr.Dataset or None
+    :param no_data: no_data value in the image
+    :type no_data: float or None
     :return: xarray.DataSet containing the variables :
             - im : 2D (row, col) xarray.DataArray float32
+    :rtype: xr.Dataset
     """
 
     data = np.copy(img_array)
@@ -345,8 +379,8 @@ def load_dems(
     dem_path: str,
     ref_nodata: float = None,
     dem_nodata: float = None,
-    ref_georef: str = "WGS84",
-    dem_georef: str = "WGS84",
+    ref_georef_grid: str = "WGS84",
+    dem_georef_grid: str = "WGS84",
     ref_geoid_path: Union[str, None] = None,
     dem_geoid_path: Union[str, None] = None,
     ref_zunit: str = "m",
@@ -356,43 +390,42 @@ def load_dems(
     """
     Loads both DEMs
 
-    :param ref_path:  path to ref dem
-    :param dem_path:path to sec dem
+    :param ref_path: path to ref dem
+    :type ref_path: str
+    :param dem_path: path to sec dem
+    :type dem_path: str
     :param ref_nodata: ref no data value
-        (None by default and if set inside metadata)
+            (None by default and if set inside metadata)
+    :type ref_nodata: float or None
     :param dem_nodata: dem no data value
-        (None by default and if set inside metadata)
-    :param ref_georef: ref georef (either WGS84 -default- or geoid)
-    :param dem_georef: dem georef (either WGS84 -default- or geoid)
+            (None by default and if set inside metadata)
+    :type dem_nodata: float or None
+    :param ref_georef_grid: ref georef (either WGS84 -default- or geoid)
+    :type ref_georef_grid: str
+    :param dem_georef_grid: dem georef (either WGS84 -default- or geoid)
+    :type dem_georef_grid: str
     :param ref_geoid_path: optional path to local geoid, default is EGM96
+    :type ref_geoid_path: str or None
     :param dem_geoid_path: optional path to local geoid, default is EGM96
+    :type dem_geoid_path: str or None
     :param ref_zunit: ref z unit
+    :type ref_zunit: str
     :param dem_zunit: dem z unit
+    :type dem_zunit: str
     :param load_data: True if dem are to be fully loaded,
-        other options are False or a dict roi
+            other options are False or a dict roi
+    :type load_data: bool, dict or Tuple
     :return: ref and dem datasets
+    :rtype: xr.Dataset, xr.Dataset
     """
 
     # Get roi of dem
-
     src_dem = rasterio.open(dem_path)
     dem_crs = src_dem.crs
 
     dem_trans = src_dem.transform
     bounds_dem = src_dem.bounds
 
-    # Reproject if input image is inversed top bottom or left right
-    # to have the same consistent (left, bottom, right, top) reference
-    # than ref projected bounds (orientation bug otherwise)
-    transformed_dem_bounds = rasterio.warp.transform_bounds(
-        dem_crs,
-        dem_crs,
-        bounds_dem[0],
-        bounds_dem[1],
-        bounds_dem[2],
-        bounds_dem[3],
-    )
-
     if load_data is not True:
         # Use ROI
         if isinstance(load_data, (tuple, list)):
@@ -426,11 +459,22 @@ def load_dems(
                     load_data["h"],
                 )
                 bounds_dem = rasterio.windows.bounds(window_dem, dem_trans)
-                print(bounds_dem)
 
             else:
                 print("Not he right conventions for ROI")
 
+    # Reproject if input image is inversed top bottom or left right
+    # to have the same consistent (left, bottom, right, top) reference
+    # than ref projected bounds (orientation bug otherwise)
+    transformed_dem_bounds = rasterio.warp.transform_bounds(
+        dem_crs,
+        dem_crs,
+        bounds_dem[0],
+        bounds_dem[1],
+        bounds_dem[2],
+        bounds_dem[3],
+    )
+
     # Get roi of ref
 
     src_ref = rasterio.open(ref_path)
@@ -481,7 +525,7 @@ def load_dems(
         new_cropped_dem_transform,
         dem_path,
         no_data=dem_nodata,
-        ref=dem_georef,
+        georef_grid=dem_georef_grid,
         geoid_path=dem_geoid_path,
         zunit=dem_zunit,
         load_data=load_data,
@@ -493,13 +537,14 @@ def load_dems(
         src_ref.transform,
         ref_path,
         no_data=ref_nodata,
-        ref=ref_georef,
+        georef_grid=ref_georef_grid,
         geoid_path=ref_geoid_path,
         zunit=ref_zunit,
         load_data=load_data,
     )
 
-    # reproject, crop, resample
+    # Reference DEM is reprojected into the sec DEM's georef-grid
+    # Crop and resample are done in the reference DEM
     ref = reproject_dataset(full_ref, dem, interp="bilinear")
     # update dataset input_img with ref old value
     ref.attrs["input_img"] = full_ref.attrs["input_img"]
@@ -514,10 +559,15 @@ def bounding_box_to_polygon(
     Transform bounding box to polygon
 
     :param left: left bound
+    :type left: float
     :param bottom: bottom bound
+    :type bottom: float
     :param right: right bound
+    :type right: float
     :param top: top bound
+    :type top: float
     :return: polygon
+    :rtype: List[List[float]]
     """
 
     polygon = [
@@ -536,10 +586,15 @@ def translate(
 ) -> xr.Dataset:
     """
     Modify transform from dataset
+
     :param dataset:
+    :type dataset: xr.Dataset
     :param x_offset: x offset
+    :type x_offset: float
     :param y_offset: y offset
-    :return translated dataset
+    :type y_offset: float
+    :return: translated dataset
+    :rtype: xr.Dataset
     """
     dataset_translated = copy.copy(dataset)
 
@@ -551,8 +606,8 @@ def translate(
 
 
 def translate_to_coregistered_geometry(
-    dem1: xr.Dataset,
-    dem2: xr.Dataset,
+    dem: xr.Dataset,
+    ref: xr.Dataset,
     dx: int,
     dy: int,
     interpolator: str = "bilinear",
@@ -561,65 +616,74 @@ def translate_to_coregistered_geometry(
     Translate both DSMs to their coregistered geometry.
 
     Note that :
-    a) The dem2 georef is assumed to be the reference
-    b) The dem2 shall be the one resampled as it supposedly is the cleaner one.
 
-    Hence, dem1 is only cropped, dem2 is the only one that might be resampled.
-    However, as dem2 is the ref, dem1 georef is translated to dem2 georef.
+    The ref georef-origin is assumed to be the reference
 
-    :param dem1: dataset, master dem
-    :param dem2: dataset, slave dem
+    The ref shall be the one resampled at dem's
+    georef-grid as it supposedly is the cleaner one.
+
+    Hence, dem is only cropped, and ref is
+    projected on dem's georef-grid, so it might be resampled.
+    However, the dem's georef-origin is translated
+    to the ref's georef-origin, which is considered the reference.
+
+    :param dem: dataset, master dem
+    :type dem: xr.Dataset
+    :param ref: dataset, slave dem
+    :type ref: xr.Dataset
     :param dx: f, dx value in pixels
+    :type dx: int
     :param dy: f, dy value in pixels
+    :type dy: int
     :param interpolator: gdal interpolator
+    :type interpolator: str
     :return: coregistered DEM as datasets
+    :rtype: xr.Dataset, xr.Dataset
     """
 
     #
-    # Translate the georef of dem1 based on dx and dy values
-    #   -> this makes dem1 coregistered on dem2
-    #
-    # note the -0.5 since the (0,0) pixel coord is pixel centered
-    dem1 = translate(dem1, dx - 0.5, dy - 0.5)
+    # Translate the georef-origin of dem based on dx and dy values
+    #   -> this makes dem coregistered on ref
+    dem = translate(dem, dx, dy)
 
     #
     # Intersect and reproject both dsms.
     #   -> intersect them to the biggest common grid
     #       now that they have been shifted
-    #   -> dem1 is then cropped with intersect so that it lies within intersect
+    #   -> dem is then cropped with intersect so that it lies within intersect
     #       but is not resampled in the process
-    #   -> reproject dem2 to dem1 grid,
-    #       the intersection grid sampled on dem1 grid
+    #   -> reproject ref to dem's georef-grid,
+    #       the intersection grid sampled on dem's grid
     #
-    transform_dem1 = Affine.from_gdal(
-        dem1["trans"].data[0],
-        dem1["trans"].data[1],
-        dem1["trans"].data[2],
-        dem1["trans"].data[3],
-        dem1["trans"].data[4],
-        dem1["trans"].data[5],
+    transform_dem = Affine.from_gdal(
+        dem["trans"].data[0],
+        dem["trans"].data[1],
+        dem["trans"].data[2],
+        dem["trans"].data[3],
+        dem["trans"].data[4],
+        dem["trans"].data[5],
     )
-    bounds_dem1 = rasterio.transform.array_bounds(
-        dem1["im"].data.shape[1], dem1["im"].data.shape[0], transform_dem1
+    bounds_dem = rasterio.transform.array_bounds(
+        dem["im"].data.shape[1], dem["im"].data.shape[0], transform_dem
     )
 
-    transform_dem2 = Affine.from_gdal(
-        dem2["trans"].data[0],
-        dem2["trans"].data[1],
-        dem2["trans"].data[2],
-        dem2["trans"].data[3],
-        dem2["trans"].data[4],
-        dem2["trans"].data[5],
+    transform_ref = Affine.from_gdal(
+        ref["trans"].data[0],
+        ref["trans"].data[1],
+        ref["trans"].data[2],
+        ref["trans"].data[3],
+        ref["trans"].data[4],
+        ref["trans"].data[5],
     )
-    bounds_dem2 = rasterio.transform.array_bounds(
-        dem2["im"].data.shape[1], dem2["im"].data.shape[0], transform_dem2
+    bounds_ref = rasterio.transform.array_bounds(
+        ref["im"].data.shape[1], ref["im"].data.shape[0], transform_ref
     )
 
     intersection_roi = (
-        max(bounds_dem1[0], bounds_dem2[0]),
-        max(bounds_dem1[1], bounds_dem2[1]),
-        min(bounds_dem1[2], bounds_dem2[2]),
-        min(bounds_dem1[3], bounds_dem2[3]),
+        max(bounds_dem[0], bounds_ref[0]),
+        max(bounds_dem[1], bounds_ref[1]),
+        min(bounds_dem[2], bounds_ref[2]),
+        min(bounds_dem[3], bounds_ref[3]),
     )
 
     # get  crop
@@ -632,35 +696,36 @@ def translate_to_coregistered_geometry(
     geom_like_polygon = {"type": "Polygon", "coordinates": [polygon_roi]}
 
     # crop dem
-    srs_dem1 = rasterio.open(
+    srs_dem = rasterio.open(
         " ",
         mode="w+",
         driver="GTiff",
-        width=dem1["im"].data.shape[1],
-        height=dem1["im"].data.shape[0],
+        width=dem["im"].data.shape[1],
+        height=dem["im"].data.shape[0],
         count=1,
-        dtype=dem1["im"].data.dtype,
-        crs=dem1.attrs["georef"],
-        transform=transform_dem1,
+        dtype=dem["im"].data.dtype,
+        crs=dem.attrs["georef"],
+        transform=transform_dem,
     )
-    srs_dem1.write(dem1["im"].data, 1)
-    new_cropped_dem1, new_cropped_dem1_transform = rasterio.mask.mask(
-        srs_dem1, [geom_like_polygon], all_touched=True, crop=True
+    srs_dem.write(dem["im"].data, 1)
+    new_cropped_dem, new_cropped_dem_transform = rasterio.mask.mask(
+        srs_dem, [geom_like_polygon], all_touched=True, crop=True
     )
 
     # create datasets
-    reproj_dem1 = copy.copy(dem1)
-    reproj_dem1["trans"].data = np.array(new_cropped_dem1_transform.to_gdal())
-    reproj_dem1 = read_img_from_array(
-        new_cropped_dem1[0, :, :],
-        from_dataset=reproj_dem1,
-        no_data=dem1.attrs["no_data"],
+    reproj_dem = copy.copy(dem)
+    reproj_dem["trans"].data = np.array(new_cropped_dem_transform.to_gdal())
+    reproj_dem = read_img_from_array(
+        new_cropped_dem[0, :, :],
+        from_dataset=reproj_dem,
+        no_data=dem.attrs["no_data"],
     )
 
-    # reproject, crop, resample
-    reproj_dem2 = reproject_dataset(dem2, reproj_dem1, interp=interpolator)
+    # Reference DEM is reprojected into the sec DEM's georef-grid
+    # Crop and resample are performed on the reference DEM
+    reproj_ref = reproject_dataset(ref, reproj_dem, interp=interpolator)
 
-    return reproj_dem1, reproj_dem2
+    return reproj_dem, reproj_ref
 
 
 def save_tif(
@@ -742,8 +807,11 @@ def get_slope(dataset: xr.Dataset, degree: bool = False) -> np.ndarray:
             fr/pro-app/tool-reference/spatial-analyst/how-aspect-works.htm
 
     :param dataset: dataset
+    :type dataset: xr.Dataset
     :param degree:  True if is in degree
+    :type degree: bool
     :return: slope
+    :rtype: np.ndarray
     """
 
     def get_orthodromic_distance(
@@ -815,10 +883,14 @@ def interpolate_geoid(
     """
     Bilinear interpolation of geoid
 
-    :param geoid_filename :  coord geoid_filename
-    :param coords :  coords matrix 2xN [lon,lat]
-    :param interpol_method  :  interpolation type
-    :return interpolated position  : [lon,lat,estimate geoid] (3D np.array)
+    :param geoid_filename: coord geoid_filename
+    :type geoid_filename: str
+    :param coords: coords matrix 2xN [lon,lat]
+    :type coords: np.ndarray
+    :param interpol_method: interpolation type
+    :type interpol_method: str
+    :return: interpolated position [lon,lat,estimate geoid]
+    :rtype: 3D np.array
     """
     dataset = rasterio.open(geoid_filename)
 
@@ -859,9 +931,12 @@ def get_geoid_offset(
     """
     Get offset from geoid to ellipsoid
 
-    :param dataset :  dataset
-    :param geoid_path :  optional absolut geoid_path, if None egm96 is used
-    :return offset as array
+    :param dataset: dataset
+    :type dataset: xr.Dataset
+    :param geoid_path: optional absolut geoid_path, if None egm96 is used
+    :type geoid_path: str or None
+    :return: offset as array
+    :rtype: np.ndarray
     """
 
     # If no geoid path has been given, use the default geoid egm96
@@ -916,3 +991,38 @@ def get_geoid_offset(
     arr_offset = np.reshape(interp_geoid, dataset["im"].data.shape)
 
     return arr_offset
+
+
+def compute_offset_bounds(
+    y_off: float, x_off: float, cfg: Dict
+) -> Tuple[float, float, float, float]:
+    """
+    Obtain the coordinate bounds to apply the offsets to
+    the secondary DEM with gdal
+
+    The offsets can be applied with the command line:
+    gdal_translate -a_ullr <ulx> <uly> <lrx> <lry>
+    /path_to_original_dem.tif /path_to_coregistered_dem.tif
+
+    :param y_off: y pixel offset
+    :type y_off: float
+    :param x_off: x pixel offset
+    :type x_off: float
+    :param cfg: cfg
+    :type cfg: Dict
+    :return: coordinate bounds to apply the offsets
+    :rtype: Tuple[float,float,float,float]
+    """
+    # Read original secondary dem
+    dem = read_img(
+        cfg["inputDSM"]["path"],
+        load_data=(
+            cfg["inputDSM"]["roi"] if "roi" in cfg["inputDSM"] else False
+        ),
+    )
+    ysize, xsize = dem["im"].shape
+    # Compute the coordinates of the new bounds
+    x_0, y_0 = pix_to_coord(dem["trans"].data, y_off, x_off)
+    x_1, y_1 = pix_to_coord(dem["trans"].data, y_off + ysize, x_off + xsize)
+
+    return float(x_0), float(y_0), float(x_1), float(y_1)
