@@ -32,10 +32,12 @@ License : MIT
 # Standard imports
 import argparse
 import os
+from typing import Tuple, Union
 
 # Third party imports
 import matplotlib.pyplot as pl
 import numpy as np
+import xarray as xr
 from scipy.interpolate import RectBivariateSpline
 from scipy.optimize import leastsq
 
@@ -43,35 +45,47 @@ from scipy.optimize import leastsq
 from .img_tools import load_dems, read_img_from_array, save_tif
 
 
-def grad2d(dem):
+def grad2d(dem: np.ndarray) -> Tuple[np.ndarray, np.ndarray]:
     """
 
-    :param dem:
+    :param dem: input dem
+    :type dem: np.ndarray
     :return: slope (fast forward style) and aspect
+    :rtype: np.ndarray, np.ndarray
     """
     grad2, grad1 = np.gradient(dem)  # in Python, x and y axis reversed
 
-    slope = np.sqrt(grad1 ** 2 + grad2 ** 2)
+    slope = np.sqrt(grad1**2 + grad2**2)
     aspect = np.arctan2(-grad1, grad2)  # aspect=0 when slope facing north
     aspect = aspect + np.pi
 
     return slope, aspect
 
 
-def nuth_kaab_single_iter(dh, slope, aspect, plot_file=None):
+def nuth_kaab_single_iter(
+    dh: np.ndarray,
+    slope: np.ndarray,
+    aspect: np.ndarray,
+    plot_file: Union[str, bool] = None,
+) -> Tuple[float, float, float]:
     """
     Compute the horizontal shift between 2 DEMs
     using the method presented in Nuth & Kaab 2011
 
-    Inputs :
-    - dh : array, elevation difference master_dem - slave_dem
-    - slope/aspect : array, slope and aspect for the same locations as the dh
-    - plot_file : file to where store plot.
-        Set to None if plot is to be printed. Set to False for no plot at all.
-    Returns :
-    - east, north, c : f, estimated easting and northing of the shift,
-        c is not used here but is related to the vertical shift
+    :param dh: elevation difference master_dem - slave_dem
+    :type dh: np.ndarray
+    :param slope: slope for the same locations as the dh
+    :type slope: np.ndarray
+    :param aspect: aspect for the same locations as the dh
+    :type aspect: np.ndarray
+    :param plot_file: file to where store plot. Set to None if
+        plot is to be printed. Set to False for no plot at all.
+    :type plot_file: str or bool
+    :return: east, north, c
+    :rtype: float, float, float
     """
+    # Compute estimated easting and northing of the shift,
+    #         c is not used here but is related to the vertical shift
     # The aim is to compute dh / tan(alpha) as a function of the aspect
     # -> hence we are going to be slice the aspect to average a value
     #     for dh / tan(alpha) on those sliced areas
@@ -80,6 +94,9 @@ def nuth_kaab_single_iter(dh, slope, aspect, plot_file=None):
     #    - b will be its orientation
     #    - c will be a vertical mean shift
 
+    # To avoid nearly-zero division, filter slope values below 0.001
+    slope[np.where(slope < 0.001)] = np.nan
+
     # function to be correlated with terrain aspect
     # NB : target = dh / tan(alpha) (see Fig. 2 of Nuth & Kaab 2011)
     # Explicitely ignore divide by zero warning,
@@ -88,39 +105,58 @@ def nuth_kaab_single_iter(dh, slope, aspect, plot_file=None):
         target = dh / slope
     target = target[np.isfinite(dh)]
     aspect = aspect[np.isfinite(dh)]
-
-    # compute median value of target for different aspect slices
-    slice_bounds = np.arange(0, 2 * np.pi, np.pi / 36)
-    mean = np.zeros([len(slice_bounds)])
-    x_s = np.zeros([len(slice_bounds)])
-    j = 0
-    for i in slice_bounds:
-        target_slice = target[
-            (i < aspect) & (aspect < i + np.pi / 36)
-        ]  # select target in the slice
-        target_slice = target_slice[
-            (target_slice < 200) & (target_slice > -200)
-        ]  # avoid target>200 and target<-200
-        mean[j] = np.median(target_slice)
-        x_s[j] = i
-        j = j + 1
+    # Define sigma to filter each target slice outliers
+    # and improve Nuth et kaab fit.
+    # All target values slice outside
+    # [mean_slice-sigma_filter*std_slice, mean_slice+sigma_filter*std_slice]
+    # are considered outliers and will be set to NaN
+    sigma_filter = 3
+    # Compute bounds for different aspect slices
+    aspect_bounds = np.arange(0, 2 * np.pi, np.pi / 36)
+    # Initialize filtered target
+    target_filt = np.full(target.shape, np.nan)
+    # Initialize slice filtered median
+    slice_filt_median = []
+    for bounds in aspect_bounds:
+        # Slice indexes within aspect
+        slice_idxes = np.where(
+            (bounds < aspect) & (aspect < bounds + np.pi / 36)
+        )
+        # If no aspect values are within the slice,
+        # fill mean with Nan and continue
+        if len(slice_idxes[0]) == 0:
+            # Set slice filtered median for Nuth et kaab as NaN
+            slice_filt_median.append(np.nan)
+            continue
+        # Obtain target values in the slice
+        target_slice = target[slice_idxes]
+        # Obtain target slice's mean and std before filtering
+        slice_mean = np.nanmean(target_slice)
+        # numpy's std cannot handle nan
+        slice_sigma = np.std(target_slice[np.isfinite(target_slice)])
+        # Target slice values outside
+        # [mean_slice-sigma_filter*std_slice, mean_slice+sigma_filter*std_slice]
+        # are considered outliers and set to NaN
+        inv_idx = np.logical_or(
+            (target_slice < (slice_mean - sigma_filter * slice_sigma)),
+            (target_slice > (slice_mean + sigma_filter * slice_sigma)),
+        )
+        # Filter target_slice
+        target_slice[inv_idx] = np.nan
+        target_slice[inv_idx] = np.nan
+        # Filter target
+        target_filt[slice_idxes] = target_slice
+        # Compute slice filtered median for Nuth et kaab
+        slice_filt_median.append(np.nanmedian(target_slice))
 
     # function to fit according to Nuth & Kaab
     x = aspect.ravel()
-    y = target.ravel()
-
+    y = target_filt.ravel()
     # remove non-finite values
-    xf = x[(np.isfinite(x)) & (np.isfinite(y))]
     yf = y[(np.isfinite(x)) & (np.isfinite(y))]
 
-    # remove outliers
-    p1 = np.percentile(yf, 1)
-    p99 = np.percentile(yf, 99)
-    xf = xf[(p1 <= yf) & (yf <= p99)]
-    yf = yf[(p1 <= yf) & (yf <= p99)]
-
     # set the first guess
-    p0 = (3 * np.std(yf) / (2 ** 0.5), 0, np.mean(yf))
+    p0 = (3 * np.std(yf) / (2**0.5), 0, np.mean(yf))
 
     # least square fit
     def peval(x, p):
@@ -134,17 +170,43 @@ def nuth_kaab_single_iter(dh, slope, aspect, plot_file=None):
 
     # we run the least square fit
     # by minimizing the "distance" between y and peval (see residuals())
-    plsq = leastsq(residuals, p0, args=(mean, x_s), full_output=1)
-    yfit = peval(x_s, plsq[0])
+    plsq = leastsq(
+        residuals, p0, args=(slice_filt_median, aspect_bounds), full_output=1
+    )
+    yfit = peval(aspect_bounds, plsq[0])
 
     # plotting results
     if plot_file is not False:
+
         pl.figure(1, figsize=(7.0, 8.0))
-        pl.plot(x_s, mean, "b.")
-        pl.plot(x_s, yfit, "k-")
-        # ax.set_ylim([np.min(mean),])
-        pl.xlabel("Terrain aspect (rad)")
-        pl.ylabel(r"dh/tan($\alpha$)")
+        pl.plot(
+            aspect * 180 / np.pi,
+            target,
+            ".",
+            color="silver",
+            markersize=3,
+            label="target",
+        )
+        pl.plot(
+            aspect * 180 / np.pi,
+            target_filt,
+            "c.",
+            markersize=3,
+            label="target filtered",
+        )
+        pl.plot(
+            aspect_bounds * 180 / np.pi, slice_filt_median, "k.", label="median"
+        )
+        pl.plot(aspect_bounds * 180 / np.pi, yfit, "b-", label="median fit")
+        pl.xlabel("Terrain aspect (deg)")
+        pl.ylabel(r"dh/tan($\alpha$) (meters)")
+        # set axes limit on twice the min/max of the median,
+        # or +-2 if the value is below it
+        axes = pl.gca()
+        ax_min = np.min([np.nanmin(slice_filt_median) * 2, -2])
+        ax_max = np.max([np.nanmax(slice_filt_median) * 2, 2])
+        axes.set_ylim([ax_min, ax_max])
+        pl.legend(loc="upper left")
         if plot_file:
             pl.savefig(plot_file, dpi=100, bbox_inches="tight")
         else:
@@ -158,7 +220,12 @@ def nuth_kaab_single_iter(dh, slope, aspect, plot_file=None):
     return east, north, c
 
 
-def nuth_kaab_lib(dsm_dataset, ref_dataset, outdir_plot=None, nb_iters=6):
+def nuth_kaab_lib(
+    dsm_dataset: xr.Dataset,
+    ref_dataset: xr.Dataset,
+    outdir_plot: str = None,
+    nb_iters: int = 6,
+) -> Tuple[float, float, float, xr.Dataset, xr.Dataset, xr.Dataset, xr.Dataset]:
     """
     This is the lib api of nuth and kaab universal coregistration.
     It offers quite the same services as the classic main api
@@ -172,11 +239,15 @@ def nuth_kaab_lib(dsm_dataset, ref_dataset, outdir_plot=None, nb_iters=6):
     NB : it is assumed that both dem3Draster have np.nan values inside
     the '.r' field as masked values
 
-    :param dsm_dataset: xarray Dataset
-    :param ref_dataset: xarray Dataset
+    :param dsm_dataset: dsm dataset
+    :type dsm_dataset: xarray Dataset
+    :param ref_dataset: ref dataset
+    :type ref_dataset: xarray Dataset
     :param outdir_plot: path to output Plot directory
-        (plots are printed if set to None)
+            (plots are printed if set to None)
+    :type outdir_plot: str
     :param nb_iters: Nuth and Kaab method iterations number default: 6
+    :type nb_iters: int
     :return: x and y shifts (as 'dsm_dataset + (x,y) = ref_dataset')
     """
 
@@ -307,9 +378,7 @@ def nuth_kaab_lib(dsm_dataset, ref_dataset, outdir_plot=None, nb_iters=6):
         "({:.2f},{:.2f})\n".format(xoff, yoff)
     )
 
-    #
-    # Get geo raster from coreg_ref array
-    #
+    # Generate datasets, use the georef-grid from the dsm
     coreg_dsm_dataset = read_img_from_array(
         coreg_dsm, from_dataset=dsm_dataset, no_data=-32768
     )
@@ -335,15 +404,15 @@ def nuth_kaab_lib(dsm_dataset, ref_dataset, outdir_plot=None, nb_iters=6):
 
 
 def run(
-    dsm_to,
-    dsm_from,
-    outfile=None,
-    nb_iters=6,
-    outdir_plot=None,
-    nan_dsm_to=None,
-    nan_dsm_from=None,
-    save_diff=False,
-):
+    dsm_to: str,
+    dsm_from: str,
+    outfile: str = None,
+    nb_iters: int = 6,
+    outdir_plot: str = None,
+    nan_dsm_to: str = None,
+    nan_dsm_from: str = None,
+    save_diff: bool = False,
+) -> Tuple[float, float, float]:
     """
     Coregister dsm_from to dsm_to using Nuth & Kaab (2011) algorithm.
 
@@ -360,16 +429,25 @@ def run(
     TODO: not used in code, Refactor with nuth_kaab_lib function.
 
     :param dsm_to: path to dsm to coregister to
+    :type dsm_to: str
     :param dsm_from: path to dsm to coregister from
+    :type dsm_from: str
     :param outfile: path to dsm_from after coregistration to dsm_to
+    :type outfile: str
     :param nb_iters: Nuth and Kaab method iterations (default(6))
+    :type nb_iters: int
     :param outdir_plot: path to output Plot directory
-        (plots are printed if set to None)
+            (plots are printed if set to None)
+    :type outdir_plot: str
     :param nan_dsm_to:
+    :type nan_dsm_to: str
     :param nan_dsm_from:
-    :param save_diff: save ./initial_dh.tiff and ./final_dh.tiff
-        with dsms diff before and after coregistration
+    :type nan_dsm_from: str
+    :param save_diff: save initial_dh, final_dh
+            and diff before and after coregistration
+    :type save_diff: bool
     :return: x and y shifts (as 'dsm_from + (x,y) = dsm_to')
+    :rtype: float, float, float
     """
 
     #
