@@ -30,19 +30,23 @@ import itertools
 import logging
 import os
 from functools import reduce
+from typing import Dict, List
 
 # Third party imports
 import matplotlib
 import matplotlib.pyplot as mpl_pyplot
 import numpy as np
+import xarray as xr
 
 # DEMcompare imports
 from .img_tools import (
     get_slope,
+    load_dems,
     read_img,
     read_img_from_array,
-    reproject_dataset,
     save_tif,
+    translate,
+    translate_to_coregistered_geometry,
 )
 from .output_tree_design import get_out_dir
 
@@ -82,14 +86,49 @@ class Partition:
     #
     def __init__(
         self,
-        name,
-        partition_kind,
-        coreg_dsm,
-        coreg_ref,
-        output_dir,
-        geo_ref=True,
-        **cfg_layer
+        name: str,
+        partition_kind: str,
+        coreg_dsm: xr.Dataset,
+        coreg_ref: xr.Dataset,
+        output_dir: str,
+        geo_ref: bool = True,
+        dec_ref_path: str = None,
+        dec_dem_path: str = None,
+        init_disp_x: int = None,
+        init_disp_y: int = None,
+        dx: float = 0.0,
+        dy: float = 0.0,
+        **cfg_layer: Dict
     ):
+        """
+        Initialization of a partition object
+
+        :param name: partition name
+        :type name: str
+        :param partition_kind: partition kind
+        :type partition_kind: str
+        :param coreg_dsm: coregistered dsm
+        :type coreg_dsm: xr.Dataset
+        :param coreg_ref: coregistered ref
+        :type coreg_ref: xr.Dataset
+        :param output_dir: output directory
+        :type output_dir: str
+        :param geo_ref: georeference
+        :type geo_ref: bool
+        :param dec_ref_path: decorelated ref path
+        :type dec_ref_path: str or None
+        :param dec_dem_path: decorelated dem path
+        :type dec_dem_path: str or None
+        :param init_disp_x: intiial disparity x
+        :type init_disp_x: int or None
+        :param init_disp_y: initial disparity y
+        :type init_disp_y: int or None
+        :param dx: Nuth offset x
+        :type dx: float or None
+        :param dy: Nuth offset y
+        :type dy: float or None
+        :return: None
+        """
 
         # Sanity check
         if partition_kind in self.type:
@@ -117,6 +156,16 @@ class Partition:
         self.coreg_path = {"ref": coreg_ref, "dsm": coreg_dsm}
         self._coreg_shape = coreg_ref["im"].data.shape
 
+        # Get coregistration offsets
+        self.dx = dx
+        self.dy = dy
+        self.init_disp_x = init_disp_x
+        self.init_disp_y = init_disp_y
+
+        # Get uncoregistred ref and dem path
+        self.dec_ref_path = dec_ref_path
+        self.dec_dem_path = dec_dem_path
+
         # Init input data path
         self.ref_path = ""
         self.dsm_path = ""
@@ -142,9 +191,10 @@ class Partition:
 
     def _create_partition_sets(self, **cfg_layer):
         """
-
-        :param cfg_layer:
-        :return:
+        Create partition sets
+        :param cfg_layer: cfg
+        :type cfg_layer: dict
+        :return: None
         """
         if self.name == "global":
             # create default partition
@@ -160,6 +210,8 @@ class Partition:
     def _create_default_partition(self):
         """
         Create default partition.
+
+        :return: None
         """
         self._sets_masks = [
             ~(
@@ -239,9 +291,12 @@ class Partition:
         return self._sets_indexes["dsm"]
 
     @property
-    def stats_results(self):
+    def stats_results(self) -> Dict:
         """
         Return stats results of partition.
+
+        :return: stats_results
+        :type stats_results: Dict
         """
         stats_results = {}
         #
@@ -280,9 +335,12 @@ class Partition:
         return stats_results
 
     @property
-    def sets_masks(self):
+    def sets_masks(self) -> List[List[np.ndarray]]:
         """
         Set masks for partition.
+
+        :return: masks set
+        :rtype: List[List[np.ndarray]]
         """
         if self._sets_masks is None:
             all_masks = []
@@ -327,9 +385,14 @@ class Partition:
             ]
         )
 
-    def generate_classes(self, ranges):
+    def generate_classes(self, ranges) -> collections.OrderedDict:
         """
         Create classes from ranges
+
+        :param ranges: ranges
+        :type ranges: List
+        :return: classes
+        :rtype: collections.OrderedDict
         """
         # change the intervals into a list to make 'classes' generic
         classes = collections.OrderedDict()
@@ -351,8 +414,7 @@ class Partition:
     def create_output_dir(self):
         """
         Create folder stats results
-        :param name_layer: layer name
-        :return:
+        :return: None
         """
         os.makedirs(self.stats_dir, exist_ok=True)
         os.makedirs(self.histograms_dir, exist_ok=True)
@@ -361,10 +423,12 @@ class Partition:
     def create_slope(self, coreg_dsm, coreg_ref):
         """
         Create slope if not exist
-        :param coreg_dsm: xarray dataset, input coregistered DSM
-        :param coreg_ref: xarray dataset, input coregistered REF
-        :param name_layer: layer name
-        :return:
+
+        :param coreg_dsm: input coregistered DSM
+        :type coreg_ds: xr.Dataset
+        :param coreg_ref: input coregistered REF
+        :type coreg_ref: xr.Dataset
+        :return: None
         """
         # Compute ref slope
         self.ref_path = os.path.join(self.stats_dir, "Ref_support.tif")
@@ -382,13 +446,16 @@ class Partition:
         )
         save_tif(slope_dsm_georaster, self.dsm_path)
 
-    def create_map(self, slope_img, type_slope):
+    def create_map(self, slope_img: str, type_slope: str):
         """
         Create the map for each slope
         (value interval is transformed into 1 value (interval minimum value)
+
         :param slope_img: slope image path
+        :type slope_img: str
         :param type_slope: type of slope : 'ref' or 'dsm'
-        :return:
+        :type type_slope: str
+        :return: None
         """
         # use radiometric ranges to classify
         slope = read_img(slope_img, load_data=False)
@@ -424,15 +491,16 @@ class Partition:
 
     def _create_set_indices(self):
         """
-        Returns a list of numpy.where, by class. Each element defines a set.
+        Returns a list of numpy.where, by class.
+        Each element defines a set.
         The sets partition / classify the image.
-        Each numpy.where contains the coordinates of the sets of the class.
+        Each numpy.where contains the coordinates
+        of the sets of the class.
         Create list of coordinates arrays :
-            -> self.sets_indices = [(label_name, np.where(...)),
-                                ... label_name, np.where(...))] ,
+        -> self.sets_indices = [(label_name,
+        np.where(...)),... label_name, np.where(...))] ,
 
-        :param classes: ordered dict of labels and associated values
-        :return:
+        :return: None
         """
         dsm_supports = ["ref", "dsm"]
         sets_indices = {support: None for support in dsm_supports}
@@ -472,7 +540,7 @@ class Partition:
 
     def _fill_sets_names_labels(self):
         """
-
+        Fills the labels of the sets names
         :return:
         """
         # fill sets_labels & sets_names
@@ -497,6 +565,7 @@ class Partition:
 
     def _fill_sets_attributes(self):
         """
+        Fills the sets attributes
 
         :return:
         """
@@ -518,7 +587,12 @@ class Partition:
             ]
 
     def _create_labelled_map(self, **cfg_layer):
-
+        """
+        Creates labelled map
+        :param cfg_layer: cfg
+        :type cfg_layer: dict
+        :return: None
+        """
         # Store classes (TODO why?)
         self._classes = {}
         if "classes" in cfg_layer:
@@ -568,21 +642,110 @@ class Partition:
 
     def rectify_map(self):
         """
-        Reproject the layer maps on top of coreg dsm and coreg ref
+        Rectify the layer maps according to coreg dsm and coreg ref
         (which are coregistered together)
 
         """
+        # TODO : make distinction when in presence of :
+        #  - other coregistration modes
+        #  - other Nuth et Kaab implementations
+        #  - other coregistration algorithms
         for map_name, map_path in self.map_path.items():
             if map_path:
                 if self.geo_ref:
-                    map_img = read_img(map_path, load_data=False)
-                    rectified_map = reproject_dataset(
-                        map_img, self.coreg_path[map_name], interp="nearest"
-                    )
-                    self.reproject_path[map_name] = os.path.join(
-                        self.stats_dir, map_name + "_support_map_rectif.tif"
-                    )
-                    save_tif(rectified_map, self.reproject_path[map_name])
+                    # If to_be_classification_layers, the map has already
+                    # been computed with coreg dsm and coreg ref
+                    if self._type_layer == "to_be_classification_layers":
+                        map_img = read_img(map_path, load_data=False)
+                        self.reproject_path[map_name] = os.path.join(
+                            self.stats_dir, map_name + "_support_map_rectif.tif"
+                        )
+                        save_tif(map_img, self.reproject_path[map_name])
+                    # If classification_layers, we need to reproject and apply
+                    # the nuth et kaab offsets for the layer to be coregistered
+                    # with coreg dsm and coreg ref
+                    if self._type_layer == "classification_layers":
+                        if map_name == "dsm":
+                            # Crop dsm
+                            ref, rectified_map_dataset = load_dems(
+                                self.dec_ref_path, map_path
+                            )
+                            # Resample images to pre-coregistered geometry
+                            # according to the initial disp
+                            if self.init_disp_x != 0 or self.init_disp_y != 0:
+                                (
+                                    rectified_map_dataset,
+                                    ref,
+                                ) = translate_to_coregistered_geometry(
+                                    rectified_map_dataset,
+                                    ref,
+                                    self.init_disp_x,
+                                    self.init_disp_y,
+                                )
+                        elif map_name == "ref":
+                            # Crop and resample ref
+                            rectified_map_dataset, dsm = load_dems(
+                                map_path, self.dec_dem_path
+                            )
+                            # Resample images to pre-coregistered geometry
+                            # according to the initial disp
+                            if self.init_disp_x != 0 or self.init_disp_y != 0:
+                                (
+                                    dsm,
+                                    rectified_map_dataset,
+                                ) = translate_to_coregistered_geometry(
+                                    dsm,
+                                    rectified_map_dataset,
+                                    self.init_disp_x,
+                                    self.init_disp_y,
+                                )
+
+                        # Update map with Nuth et kaab offsets
+                        map_img = rectified_map_dataset.copy()
+                        if self.dx >= 0:
+                            rectified_map = rectified_map_dataset["im"].data[
+                                :,
+                                0 : map_img["im"].data.shape[1]
+                                - int(np.ceil(self.dx)),
+                            ]
+                        else:
+                            rectified_map = rectified_map_dataset["im"].data[
+                                :,
+                                int(np.floor(-self.dx)) : map_img[
+                                    "im"
+                                ].data.shape[1],
+                            ]
+                        if -self.dy >= 0:
+                            rectified_map = rectified_map[
+                                0 : map_img["im"].data.shape[0]
+                                - int(np.ceil(-self.dy)),
+                                :,
+                            ]
+                        else:
+                            rectified_map = rectified_map[
+                                int(np.floor(self.dy)) : map_img[
+                                    "im"
+                                ].data.shape[0],
+                                :,
+                            ]
+                        # Generate dataset
+                        rectified_map_dataset = read_img_from_array(
+                            rectified_map, from_dataset=map_img, no_data=-32768
+                        )
+                        # Translate the georef-origin of rectified_map based
+                        # on x_off and y_off values
+                        #   -> this makes dem coregistered on ref
+                        rectified_map_dataset = translate(
+                            rectified_map_dataset, self.dx, -self.dy
+                        )
+
+                        self.reproject_path[map_name] = os.path.join(
+                            self.stats_dir, map_name + "_support_map_rectif.tif"
+                        )
+
+                        save_tif(
+                            rectified_map_dataset, self.reproject_path[map_name]
+                        )
                 else:
                     self.reproject_path[map_name] = map_path
 
@@ -639,14 +802,18 @@ class FusionPartition(Partition):
 
     def _create_partition_sets(self, **_):
         """
-
+        Creates partitions sets
         :param kwargs:
-        :return:
+        :return: None
         """
         self._fill_sets_attributes()
         self._set_labelled_map()
 
     def _set_labelled_map(self):
+        """
+        Sets labelled map
+        :return: None
+        """
         for (
             df_k,
             df_v,
@@ -672,6 +839,10 @@ class FusionPartition(Partition):
                 )
 
     def _fill_sets_attributes(self):
+        """
+        Fills sets attributes
+        :return: None
+        """
         all_combi_labels, self._classes = self._create_merged_classes(
             self.partitions
         )
@@ -706,11 +877,12 @@ class FusionPartition(Partition):
                                 label_idx
                             ]
                         )
-
+                    for indexes2d in all_indexes:
+                        np.ravel_multi_index(indexes2d, self._coreg_shape)
                     # ravel indexes so we can merge them
                     all_indexes = [
-                        np.ravel_multi_index(indexes2D, self._coreg_shape)
-                        for indexes2D in all_indexes
+                        np.ravel_multi_index(indexes2d, self._coreg_shape)
+                        for indexes2d in all_indexes
                     ]
 
                     # merge indexes and unravel them
@@ -763,6 +935,8 @@ class FusionPartition(Partition):
 def get_color(nb_color=10):
     """
     Function to get matplotlib color possibilities.
+    :param nb_color: number of colors
+    :type nb_color: int
     """
 
     if 10 < nb_color < 21:
@@ -786,13 +960,22 @@ def get_color(nb_color=10):
     return np.array(x.colors[0:nb_color])
 
 
-def create_fusion(sets_masks, all_combi_labels, classes_fusion, layers_obj):
+def create_fusion(
+    sets_masks: Dict, all_combi_labels, classes_fusion, layers_obj
+):
     """
     TODO: create all maps fusion
     :param sets_masks: dict per layer (example 'slope', 'carte_occupation', ...)
-        contains a tuple list each,
-        where each tuple contains ('label_name', A3DGeoRaster_mask)
-    layers_obj: example layer to get size and georef
+    contains a tuple list each,
+    where each tuple contains ('label_name', A3DGeoRaster_mask)
+    :type sets_masks: Dict
+    :param all_combi_labels:
+    :type all_combi_labels: List
+    :param classes_fusion:
+    :type classes_fusion: List
+    :param layers_obj: example layer to get size and georef
+    :type layers_obj:
+    layers_obj:
     :return:
     """
     # create map which fusion all classes combinaisons
